import json, os

from . import xdg

class Cache():
    def __init__(self):
        # init XDG path to store in
        self.dump_file = xdg.initXDG('influxtap') + 'cache.json'

# dump provided datapoints cache data to a file
    def dump(self, datapoints):
        if len(datapoints) > 0:
            with open(self.dump_file, 'w') as fh:
                json.dump(datapoints, fh)
                return True
        else:
            return False

    def read(self):
        if self.exists():
            with open(self.dump_file, 'r') as fh:
                return json.load(fh)
        else:
            return None

    def clear(self):
        if self.exists():
            os.unlink(self.dump_file)
            return True
        else:
            return False

    def flush(self):
        out = read()
        clear()
        return out

    def exists(self):
        return os.path.isfile(self.dump_file)



#!/usr/bin/env python3

import yaml
import requests, datetime
from influxdb import InfluxDBClient, SeriesHelper
from influxdb.exceptions import InfluxDBServerError
from urllib3.exceptions import NewConnectionError
from requests.exceptions import ConnectionError
from absl import flags, logging

from . import cache

FLAGS = flags.FLAGS


""" Low level InfluxDB series writing helper
"""
class RequestSeries(SeriesHelper):
    class Meta:
        series_name = 'influxtap.request'
        fields = ['status_code', 'elapsed']
        tags = ['url', 'request_type' ]

""" Main class for running the individual requests and storing their results
"""
class Tappery():
    def __init__(self, config):
        self.requests = [] # results queue for storage
        self.datapoints = [] # queue for datapoints going into storage
        self.cache = cache.Cache() # outgoing write cache handling
        self.reload_config(config)
        if self.cache.exists(): # populate any existing cache dump
            self.datapoints.extend(cache.read())


    """ High-level storage of a list of requests.result objects
    """
    def store(self):
        dps = []
        for result in self.requests:
            dp = {
                'status_code': result.status_code,
                'request_type': 'GET', # TODO(artanicus): support other request types, perhaps
                'url': result.request.url,
                'elapsed': result.elapsed / datetime.timedelta(milliseconds=1) # store elapsed in milliseconds
                }
            logging.info('[{0}] {1} in {2}'.format(result.status_code, result.request.url, result.elapsed))
            dps.append(dp)
        # push to write queue
        self.datapoints.extend(dps)
        # clear read queue
        self.requests = []
        try:
            logging.info('writing to InfluxDB...')
            self._write_influx(self.datapoints)
        except InfluxDBServerError:
            logging.warning('Data kept in cache({0}), issues writing to InfluxDB'.format(len(self.datapoints)))
        else:
            # write succeeded, drop cache
            logging.info('Write({0}) successful!'.format(len(self.datapoints)))
            self.datapoints = [] # clear temporary cache
            self.cache.clear() # clear any permanent cache we may have had

    """ Do a round of requests and store the results
    """
    def probe(self):
        for url in self.urls:
            try:
                self.requests.append(requests.get(url))
            except (ConnectionError, NewConnectionError) as e:
                logging.error('Connection failed [{0}]: {1}'.format(url, e))

    """ InfluxDB writer from a list of well formatted datapoints
        - Timestamps are assumed to be in UTC unless overriden with tz
    """
    def _write_influx(self, datapoints):
        assert(len(datapoints) > 0)
        for dp in datapoints:
            RequestSeries(**dp)
        RequestSeries.commit(self.db)

    def reload_config(self, config):
        with open(config) as fp:
            config = yaml.load(fp, Loader=yaml.FullLoader)
        self.urls = config.get('urls')
        self.influxdb_config = config.get('influxdb')
        self.db = InfluxDBClient(**self.influxdb_config)



import configparser
import os

def initXDG(program_name):
    # per the XDG basedir-spec we adhere to $XDG_CONFIG_HOME if it's set, otherwise assume $HOME/.config
    xdg_config_home = ''
    if 'XDG_CONFIG_HOME' in os.environ:
        xdg_config_home = os.environ['XDG_CONFIG_HOME']
    else:
        xdg_config_home = "%s/.config" % os.path.expanduser('~')

    # XDG base-dir: "If, when attempting to write a file, the destination directory is non-existant an attempt should be made to create it with permission 0700. If the destination directory exists already the permissions should not be changed."
    if not os.path.isdir(xdg_config_home):
        os.mkdir(xdg_config_home, 0o0700)

    # finally create our own config dir
    config_dir = "%s/%s" % (xdg_config_home, program_name)
    if not os.path.isdir(config_dir):
        os.mkdir(config_dir, 0o0700)

    return config_dir + '/'



from setuptools import setup

setup(
        name='influxtap',
        description='Daemon-like utility to monitor website responses and push the data into InfluxDB.',
        license='MIT',
        version='0.1.0',
        python_requires='>=3.0',
        author='artanicus',
        author_email='influxtap@nocturnal.fi',
        url='https://github.com/Artanicus/influxtap',
        packages=['influxtap'],
        install_requires=['requests', 'pyyaml', 'absl-py', 'influxdb'],
        entry_points={
          'console_scripts': [
              'influxtap=influxtap.__main__:main'
          ]
        },
      )




#!/usr/bin/env python3
import signal, time
from absl import flags, logging, app

import influxtap

FLAGS = flags.FLAGS
logging.set_verbosity(logging.WARNING)

flags.DEFINE_string('config', None, 'Path to YAML formatted config file.')
flags.DEFINE_integer('interval', 60, 'Global interval for data collection in seconds.')

flags.mark_flag_as_required('config')
    

def real_main(argv = None):
    tap = influxtap.Tappery(FLAGS.config)
    
    # Handle cache dumping if killed.
    def sigterm_handler(signal, frame):
        tap.cache.dump()
        logging.fatal('SIGTERM, going down')

    # Handle config reload on HUP.
    def sighup_handler(signal, frame):
        logging.warning('SIGHUP, reloading config')
        tap.reload_config(FLAGS.config)
    
    signal.signal(signal.SIGTERM, sigterm_handler)
    signal.signal(signal.SIGHUP, sighup_handler)
    
    while True:
        tap.probe()
        tap.store()
        time.sleep(FLAGS.interval)


def main(argv = None):
    app.run(real_main)

if __name__ == "__main__":
    app.run(real_main)
